/* Copyright (c) 2014 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "sdk_common.h"
#include "nrf.h"
#include "nrf_esb_error_codes.h"
#include "nrf_delay.h"
#include "nrf_gpio.h"
#include "nrf_error.h"
#include "app_error.h"
#include "boards.h"
#include "SEGGER_RTT.h"
#include "nrf_drv_clock.h"
#include "app_timer.h"
#include "app_button.h"

#include "radio.h"
#include "display.h"
#include "gps.h"


#define NRF_LOG_MODULE_NAME "APP"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"

//set VDD to 3.3V
const uint32_t UICR_ADDR_0x304 __attribute__((at(0x10001304))) __attribute__((used)) = 0x5;

#define LED1 13
#define LED2 14
#define LED3 15
#define LED4 16

#define GPS_UART_RX (32+9)
#define RECEIVE_LED LED2

#define NUM_OF_BUTTONS 4
#define BUTTON1 11
#define BUTTON2 12
#define BUTTON3 24
#define BUTTON4 25

// General application timer settings.
#define APP_TIMER_PRESCALER             15    // Value of the RTC1 PRESCALER register.
#define APP_TIMER_OP_QUEUE_SIZE         3     // Size of timer operation queues.

static radio_modes_t mode = MODE_125_KBIT;
static int8_t tx_power = 9;

const uint8_t leds_list[LEDS_NUMBER] = LEDS_LIST;
uint8_t led_nr;

//if this is true, the application will use a static position and not the one retrieved from the GPS
static bool test = true;
static char gps_test[128] = "000000.000,6325.3812,N,01026.3644,E,1,08,1.12,110.9";

static char gps_local[128];

static uint8_t packet[128];

void HardFault_Handler(void)
{
    uint32_t *sp = (uint32_t *) __get_MSP(); // Get stack pointer
    uint32_t ia = sp[24/4]; // Get instruction address from stack

    SEGGER_RTT_printf(0,"Hard Fault at address: 0x%08x\r\n", (unsigned int)ia);
    
    nrf_gpio_pin_clear(LED1);
    nrf_gpio_pin_clear(LED2);
    nrf_gpio_pin_clear(LED3);
    nrf_gpio_pin_clear(LED4);
    
    while(1)
        ;
}

static void button_event_handler(uint8_t pin_no, uint8_t button_action)
{
    if(button_action == APP_BUTTON_PUSH)
    {
        switch(pin_no)
        {
            case BUTTON1:
                nrf_gpio_pin_toggle(LED4);
                if(mode == MODE_125_KBIT)
                {
                    mode = MODE_1_MBIT;
                    //display_data("", "", 0, mode, packet[0]);
                }
                else
                {
                    mode = MODE_125_KBIT;
                    //display_data("", "", 0, mode, packet[0]);
                }
                radio_set_mode(mode);
                break;

            case BUTTON2:
                break;
                
            case BUTTON3:
                test = !test;
                break;
            
            case BUTTON4:
                break;
                
            default:
                break;
        }
    }
}

static const app_button_cfg_t app_buttons[NUM_OF_BUTTONS] = 
{
    {BUTTON1, false, BUTTON_PULL, button_event_handler},
    {BUTTON2, false, BUTTON_PULL, button_event_handler},
    {BUTTON3, false, BUTTON_PULL, button_event_handler},
    {BUTTON4, false, BUTTON_PULL, button_event_handler},
};

static void lfclk_request(void)
{
    uint32_t err_code = nrf_drv_clock_init();
    APP_ERROR_CHECK(err_code);
    nrf_drv_clock_lfclk_request(NULL);
}

static void leds_init()
{
    nrf_gpio_cfg_output(LED1);
    nrf_gpio_cfg_output(LED2);
    nrf_gpio_cfg_output(LED3);
    nrf_gpio_cfg_output(LED4);
    
    nrf_gpio_pin_set(LED1);
    nrf_gpio_pin_set(LED2);
    nrf_gpio_pin_set(LED3);
    nrf_gpio_pin_set(LED4);
}

int main(void)
{   
    uint32_t err_code;
    
    //SEGGER_RTT_printf(0, "start\n");

    leds_init();
    
    lfclk_request();
    
    //app_button uses app_timer, if this is not initialize, then initialize it here
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, false);
    
    //init app_button module, 10ms detection delay (button debouncing)
    err_code = app_button_init((app_button_cfg_t *)app_buttons,
                                   NUM_OF_BUTTONS,
                                   APP_TIMER_TICKS(10, APP_TIMER_PRESCALER));
    APP_ERROR_CHECK(err_code);

    err_code = app_button_enable();
    APP_ERROR_CHECK(err_code);
    
    uart_init(GPS_UART_RX);
    
    display_init();
    
    err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    radio_setup(false, tx_power, mode);
    radio_receive_packet();
    
    while (true)
    {
        __SEV();
        __WFE();
        __WFE();
        if(is_packet_received())
        {
                nrf_gpio_pin_toggle(RECEIVE_LED);
                int8_t rssi = get_rssi();
                
                radio_get_packet(packet);
                
                if(test)
                {
                    display_data(&packet[1], gps_test, rssi, mode, packet[0], test);
                }
                else
                {
                    get_gps_local(gps_local);
                    display_data(&packet[1], gps_local, rssi, mode, packet[0], test);
                }
        }
    }
}
